require 'sketchup'

module Matt_Tools
  module Toggle_Units
    module_function

    @dir = File.dirname(__FILE__)
    @strings = {}

    # Load language strings
    lang_file = if ['cn', 'CN', 'zh-CN'].include?(Sketchup.os_language)
                 File.join(@dir, 'tustrings.cn')
               else
                 File.join(@dir, 'tustrings.en')
               end

    # Check if the language file exists
    if File.exist?(lang_file)
      f = IO.readlines(lang_file)
      @strings = {} # Ensure @strings is initialized
      f.each do |l|
        l = l.strip.split('___')
        @strings[l.first] = l[1] if l.size >= 2 # Ensure the line has at least two parts
      end
    else
      UI.messagebox("Language file not found: #{lang_file}")
      # Fallback to default values
      @strings = {
        'UnitsOptions' => 'UnitsOptions',
        'LengthUnit' => 'LengthUnit',
        'LengthFormat' => 'LengthFormat',
        'LengthPrecision' => 'LengthPrecision',
        'SuppressUnitsDisplay' => 'SuppressUnitsDisplay',
        'tufe' => 'Feet',
        'tuin' => 'Inches',
        'tumm' => 'Millimeters',
        'tucm' => 'Centimeters',
        'tume' => 'Meters',
        'tudec' => 'Decimal',
        'tuarc' => 'Architectural',
        'tufra' => 'Fractional',
        'tuing' => 'Engineering',
        'lu' => 'Length unit set to',
        'lf' => 'Length format set to',
        'dp' => 'Decimal precision set to',
        'df' => 'Display format toggled',
        'hf' => 'Hide format toggled',
        'ddp' => 'Decrease Decimal Precision',
        'idp' => 'Increase Decimal Precision',
        'tm' => 'Toolbar Manager',
        'atc' => 'Changes applied successfully!',
        'ntc' => 'No changes were made.'
      }
    end

    def go(int)
      model = Sketchup.active_model
      options = model.options
      units_options = options['UnitsOptions']

      # Check if units_options is nil
      if units_options.nil?
        UI.messagebox("UnitsOptions is not available. Please check your SketchUp model.")
        return
      end

      lst = {
        0 => @strings['tufe'],
        1 => @strings['tuin'],
        2 => @strings['tumm'],
        3 => @strings['tucm'],
        4 => @strings['tume'],
        5 => @strings['tudec'],
        6 => @strings['tuarc'],
        7 => @strings['tufra'],
        8 => @strings['tuing']
      }

      case int
      when 0, 1, 2, 3, 4
        # Update the length unit
        units_options['LengthUnit'] = int
        puts "#{@strings['lu']} #{lst[int]}"
      when 5, 6, 7, 8
        # Update the length format
        result = int - 5
        units_options['LengthFormat'] = result
        puts "#{@strings['lf']} #{lst[int]}"
      end

      # Force toolbar refresh to update button states
      refresh_toolbar
      model.active_view.invalidate
    end

    def validation_proc(int)
      model = Sketchup.active_model
      options = model.options
      units_options = options['UnitsOptions']

      # Check if units_options is nil
      if units_options.nil?
        return MF_GRAYED # Disable the button if units_options is nil
      end

      case int
      when 0, 1, 2, 3, 4
        # Check if this button is the current length unit
        if units_options['LengthUnit'] == int
          MF_CHECKED
        else
          # Disable non-active unit buttons based on format
          case units_options['LengthFormat']
          when 1 # Architectural (inches)
            if int != 1 # 1 is inches
              MF_GRAYED
            else
              MF_UNCHECKED
            end
          when 2 # Fractional (feet)
            if int != 0 # 0 is feet
              MF_GRAYED
            else
              MF_UNCHECKED
            end
          when 3 # Engineering (inches)
            if int != 1 # 1 is inches
              MF_GRAYED
            else
              MF_UNCHECKED
            end
          else
            MF_UNCHECKED
          end
        end
      when 5, 6, 7, 8
        # Check if this button is the current length format
        if units_options['LengthFormat'] == (int - 5)
          MF_CHECKED
        else
          MF_UNCHECKED
        end
      end
    end

    def refresh_toolbar
      # Force the toolbar to refresh its buttons
      if @toolbar
        @toolbar.refresh
      end
    end

    def Length_Precision(int)
      model = Sketchup.active_model
      options = model.options
      units_options = options['UnitsOptions']

      # Check if units_options is nil
      if units_options.nil?
        UI.messagebox("UnitsOptions is not available. Please check your SketchUp model.")
        return
      end

      return unless [1, -1].include?(int)
      lp = units_options['LengthPrecision']
      if 0 <= (lp + int) && (lp + int) <= 10
        units_options['LengthPrecision'] = lp + int
        puts "#{@strings['dp']} #{lp + int}"
      end
      model.active_view.invalidate
    end

    def display_unit_format
      model = Sketchup.active_model
      options = model.options
      units_options = options['UnitsOptions']

      # Check if units_options is nil
      if units_options.nil?
        UI.messagebox("UnitsOptions is not available. Please check your SketchUp model.")
        return
      end

      if units_options['SuppressUnitsDisplay'] == true
        units_options['SuppressUnitsDisplay'] = false
        puts @strings['df']
      else
        units_options['SuppressUnitsDisplay'] = true
        puts @strings['hf']
      end
      model.active_view.invalidate
    end

    def display_unit_format_validation_proc
      model = Sketchup.active_model
      options = model.options
      units_options = options['UnitsOptions']

      # Check if units_options is nil
      if units_options.nil?
        return MF_GRAYED # Disable the button if units_options is nil
      end

      if [1, 2].include?(units_options['LengthFormat'])
        MF_GRAYED
      elsif units_options['SuppressUnitsDisplay'] == true
        MF_UNCHECKED
      else
        MF_CHECKED
      end
    end

    def toolbar_manager
      puts 'Matt_Tools::Toggle_Units.toolbar_manager'
      @dir = File.dirname(__FILE__)
      settings_file = File.join(@dir, 'settings.txt')

      if File.exist?(settings_file)
        settings = IO.readlines(settings_file)
        settings.each { |l| l.strip! }
      else
        UI.messagebox("Settings file not found: #{settings_file}")
        settings = Array.new(14, 'true') # Fallback to default settings
      end

      initial_settings = settings.dup

      dialog = UI::HtmlDialog.new(
        {
          dialog_title: @strings['tm'],
          preferences_key: 'toggle_units_toolbar_manager',
          width: 320,
          height: 580,
          resizable: false
        }
      )
      dialog.set_file(File.join(@dir, 'toolbar_manager.html'))

      dialog.add_action_callback('cback') do |_, params|
        case params
        when 'stop'
          dialog.close
        when 'go'
          if settings != initial_settings
            File.open(settings_file, 'w') do |f|
              settings.each { |e| f.puts e }
            end
            UI.messagebox(@strings['atc'])
          else
            UI.messagebox(@strings['ntc'])
          end
          dialog.close
        when '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10', '11', '12', '13'
          n = params.to_i
          settings[n] = (settings[n] == 'true') ? 'false' : 'true'
        end
      end

      dialog.show
    end

    unless file_loaded?(__FILE__)
      file_loaded(__FILE__)

      $submenu ? (organizer = $submenu) : (organizer = UI.menu('Plugins'))
      menu = organizer.add_submenu(@strings['tu'])
      menu.add_item(@strings['tudec']) { go(5) }
      menu.add_item(@strings['tuarc']) { go(6) }
      menu.add_item(@strings['tuing']) { go(8) }  # Swapped third and fourth buttons
      menu.add_item(@strings['tufra']) { go(7) }  # Swapped third and fourth buttons
      menu.add_separator
      menu.add_item(@strings['tuin']) { go(1) }
      menu.add_item(@strings['tufe']) { go(0) }
      menu.add_item(@strings['tumm']) { go(2) }
      menu.add_item(@strings['tucm']) { go(3) }
      menu.add_item(@strings['tume']) { go(4) }
      menu.add_separator
      menu.add_item(@strings['ddp']) { Length_Precision(-1) }
      menu.add_item(@strings['idp']) { Length_Precision(1) }
      menu.add_item(@strings['df']) { display_unit_format }
      menu.add_separator
      menu.add_item(@strings['tm']) { toolbar_manager }

      settings_file = File.join(@dir, 'settings.txt')
      settings = if File.exist?(settings_file)
                  IO.readlines(settings_file)
                else
                  Array.new(14, 'true') # Fallback to default settings
                end
      settings.each { |l| l.strip! }

      @toolbar = UI::Toolbar.new(@strings['tu'])
      n = 0
      while n < settings.length
        if settings[n] == 'true'
          case n
          when 0
            cmd = UI::Command.new(@strings['tudec']) { go(5) }
            cmd.small_icon = File.join(@dir, 'tudec_16.png')
            cmd.large_icon = File.join(@dir, 'tudec_24.png')
            cmd.tooltip = cmd.status_bar_text = "#{@strings['tudu']} #{@strings['tudec']}"
            cmd.set_validation_proc { validation_proc(5) }
            @toolbar.add_item cmd
          when 1
            cmd = UI::Command.new(@strings['tuarc']) { go(6) }
            cmd.small_icon = File.join(@dir, 'tuarc_16.png')
            cmd.large_icon = File.join(@dir, 'tuarc_24.png')
            cmd.tooltip = cmd.status_bar_text = "#{@strings['tudu']} #{@strings['tuarc']}"
            cmd.set_validation_proc { validation_proc(6) }
            @toolbar.add_item cmd
          when 2
            cmd = UI::Command.new(@strings['tuing']) { go(8) }  # Swapped third and fourth buttons
            cmd.small_icon = File.join(@dir, 'tuing_16.png')
            cmd.large_icon = File.join(@dir, 'tuing_24.png')
            cmd.tooltip = cmd.status_bar_text = "#{@strings['tudu']} #{@strings['tuing']}"
            cmd.set_validation_proc { validation_proc(8) }
            @toolbar.add_item cmd
          when 3
            cmd = UI::Command.new(@strings['tufra']) { go(7) }  # Swapped third and fourth buttons
            cmd.small_icon = File.join(@dir, 'tufra_16.png')
            cmd.large_icon = File.join(@dir, 'tufra_24.png')
            cmd.tooltip = cmd.status_bar_text = "#{@strings['tudu']} #{@strings['tufra']}"
            cmd.set_validation_proc { validation_proc(7) }
            @toolbar.add_item cmd
          when 4
            @toolbar.add_separator
          when 5
            cmd = UI::Command.new(@strings['tuin']) { go(1) }
            cmd.small_icon = File.join(@dir, 'tuin_16.png')
            cmd.large_icon = File.join(@dir, 'tuin_24.png')
            cmd.tooltip = cmd.status_bar_text = "#{@strings['tudu']} #{@strings['tuin']}"
            cmd.set_validation_proc { validation_proc(1) }
            @toolbar.add_item cmd
          when 6
            cmd = UI::Command.new(@strings['tufe']) { go(0) }
            cmd.small_icon = File.join(@dir, 'tufe_16.png')
            cmd.large_icon = File.join(@dir, 'tufe_24.png')
            cmd.tooltip = cmd.status_bar_text = "#{@strings['tudu']} #{@strings['tufe']}"
            cmd.set_validation_proc { validation_proc(0) }
            @toolbar.add_item cmd
          when 7
            cmd = UI::Command.new(@strings['tumm']) { go(2) }
            cmd.small_icon = File.join(@dir, 'tumm_16.png')
            cmd.large_icon = File.join(@dir, 'tumm_24.png')
            cmd.tooltip = cmd.status_bar_text = "#{@strings['tudu']} #{@strings['tumm']}"
            cmd.set_validation_proc { validation_proc(2) }
            @toolbar.add_item cmd
          when 8
            cmd = UI::Command.new(@strings['tucm']) { go(3) }
            cmd.small_icon = File.join(@dir, 'tucm_16.png')
            cmd.large_icon = File.join(@dir, 'tucm_24.png')
            cmd.tooltip = cmd.status_bar_text = "#{@strings['tudu']} #{@strings['tucm']}"
            cmd.set_validation_proc { validation_proc(3) }
            @toolbar.add_item cmd
          when 9
            cmd = UI::Command.new(@strings['tume']) { go(4) }
            cmd.small_icon = File.join(@dir, 'tume_16.png')
            cmd.large_icon = File.join(@dir, 'tume_24.png')
            cmd.tooltip = cmd.status_bar_text = "#{@strings['tudu']} #{@strings['tume']}"
            cmd.set_validation_proc { validation_proc(4) }
            @toolbar.add_item cmd
          when 10
            @toolbar.add_separator
          when 11
            cmd = UI::Command.new(@strings['ddp']) { Length_Precision(-1) }
            cmd.small_icon = File.join(@dir, 'tumoins_16.png')
            cmd.large_icon = File.join(@dir, 'tumoins_24.png')
            cmd.tooltip = cmd.status_bar_text = @strings['ddp']
            @toolbar.add_item cmd
          when 12
            cmd = UI::Command.new(@strings['idp']) { Length_Precision(1) }
            cmd.small_icon = File.join(@dir, 'tuplus_16.png')
            cmd.large_icon = File.join(@dir, 'tuplus_24.png')
            cmd.tooltip = cmd.status_bar_text = @strings['idp']
            @toolbar.add_item cmd
          when 13
            cmd = UI::Command.new(@strings['df']) { display_unit_format }
            cmd.small_icon = File.join(@dir, 'tuduf_16.png')
            cmd.large_icon = File.join(@dir, 'tuduf_24.png')
            cmd.tooltip = cmd.status_bar_text = @strings['df']
            cmd.set_validation_proc { display_unit_format_validation_proc }
            @toolbar.add_item cmd
          end
        end
        n += 1
      end

      # Restore or show toolbar
      state = @toolbar.visible?
      if state
        @toolbar.restore
      else
        @toolbar.show
      end
    end
  end
end